_ATHANOR_RIGS = {
    "Standup M-Set Biochemical Reactor Material Efficiency I",
    "Standup M-Set Biochemical Reactor Time Efficiency I",
    "Standup M-Set Composite Reactor Material Efficiency I",
    "Standup M-Set Composite Reactor Time Efficiency I",
    "Standup M-Set Hybrid Reactor Material Efficiency I",
    "Standup M-Set Hybrid Reactor Time Efficiency I",
    "Standup M-Set Biochemical Reactor Material Efficiency II",
    "Standup M-Set Biochemical Reactor Time Efficiency II",
    "Standup M-Set Composite Reactor Material Efficiency II",
    "Standup M-Set Composite Reactor Time Efficiency II",
    "Standup M-Set Hybrid Reactor Material Efficiency II",
    "Standup M-Set Hybrid Reactor Time Efficiency II",
    "Standup M-Set Advanced Component Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Component Manufacturing Time Efficiency I",
    "Standup M-Set Advanced Large Ship Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Large Ship Manufacturing Time Efficiency I",
    "Standup M-Set Advanced Medium Ship Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Medium Ship Manufacturing Time Efficiency I",
    "Standup M-Set Advanced Small Ship Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Small Ship Manufacturing Time Efficiency I",
    "Standup M-Set Ammunition Manufacturing Material Efficiency I",
    "Standup M-Set Ammunition Manufacturing Time Efficiency I",
    "Standup M-Set Basic Capital Component Manufacturing Material Efficiency I",
    "Standup M-Set Basic Capital Component Manufacturing Time Efficiency I",
    "Standup M-Set Basic Large Ship Manufacturing Material Efficiency I",
    "Standup M-Set Basic Large Ship Manufacturing Time Efficiency I",
    "Standup M-Set Basic Medium Ship Manufacturing Material Efficiency I",
    "Standup M-Set Basic Medium Ship Manufacturing Time Efficiency I",
    "Standup M-Set Basic Small Ship Manufacturing Material Efficiency I",
    "Standup M-Set Basic Small Ship Manufacturing Time Efficiency I",
    "Standup M-Set Drone and Fighter Manufacturing Material Efficiency I",
    "Standup M-Set Drone and Fighter Manufacturing Time Efficiency I",
    "Standup M-Set Equipment Manufacturing Material Efficiency I",
    "Standup M-Set Equipment Manufacturing Time Efficiency I",
    "Standup M-Set Structure Manufacturing Material Efficiency I",
    "Standup M-Set Structure Manufacturing Time Efficiency I",
    "Standup M-Set Thukker Advanced Component Manufacturing Material Efficiency",
    "Standup M-Set Thukker Basic Capital Component Manufacturing Material Efficiency",
    "Standup M-Set Advanced Component Manufacturing Material Efficiency II"
}

_TATARA_RIGS = {
    "Standup L-Set Reactor Efficiency I",
    "Standup L-Set Reactor Efficiency II",
    "Standup L-Set Advanced Component Manufacturing Efficiency I",
    "Standup L-Set Advanced Large Ship Manufacturing Efficiency I",
    "Standup L-Set Advanced Medium Ship Manufacturing Efficiency I",
    "Standup L-Set Advanced Small Ship Manufacturing Efficiency I",
    "Standup L-Set Ammunition Manufacturing Efficiency I",
    "Standup L-Set Basic Capital Component Manufacturing Efficiency I",
    "Standup L-Set Basic Large Ship Manufacturing Efficiency I",
    "Standup L-Set Basic Medium Ship Manufacturing Efficiency I",
    "Standup L-Set Basic Small Ship Manufacturing Efficiency I",
    "Standup L-Set Capital Ship Manufacturing Efficiency I",
    "Standup L-Set Drone and Fighter Manufacturing Efficiency I",
    "Standup L-Set Equipment Manufacturing Efficiency I",
    "Standup L-Set Structure Manufacturing Efficiency I",
    "Standup L-Set Thukker Advanced Component Manufacturing Efficiency",
    "Standup L-Set Thukker Basic Capital Component Manufacturing Efficiency",
    "Standup L-Set Advanced Component Manufacturing Efficiency II",
    "Standup L-Set Advanced Large Ship Manufacturing Efficiency II",
    "Standup L-Set Advanced Medium Ship Manufacturing Efficiency II",
    "Standup L-Set Advanced Small Ship Manufacturing Efficiency II",
    "Standup L-Set Capital Ship Manufacturing Efficiency II",
    "Standup L-Set Ammunition Manufacturing Efficiency II",
    "Standup L-Set Basic Capital Component Manufacturing Efficiency II",
    "Standup L-Set Basic Large Ship Manufacturing Efficiency II",
    "Standup L-Set Basic Medium Ship Manufacturing Efficiency II",
    "Standup L-Set Basic Small Ship Manufacturing Efficiency II",
    "Standup L-Set Drone and Fighter Manufacturing Efficiency II",
    "Standup L-Set Equipment Manufacturing Efficiency II",
    "Standup L-Set Structure Manufacturing Efficiency II"
}

_RAITARU_RIGS = {
    "Standup M-Set Advanced Component Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Component Manufacturing Time Efficiency I",
    "Standup M-Set Advanced Large Ship Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Large Ship Manufacturing Time Efficiency I",
    "Standup M-Set Advanced Medium Ship Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Medium Ship Manufacturing Time Efficiency I",
    "Standup M-Set Advanced Small Ship Manufacturing Material Efficiency I",
    "Standup M-Set Advanced Small Ship Manufacturing Time Efficiency I",
    "Standup M-Set Ammunition Manufacturing Material Efficiency I",
    "Standup M-Set Ammunition Manufacturing Time Efficiency I",
    "Standup M-Set Basic Capital Component Manufacturing Material Efficiency I",
    "Standup M-Set Basic Capital Component Manufacturing Time Efficiency I",
    "Standup M-Set Basic Large Ship Manufacturing Material Efficiency I",
    "Standup M-Set Basic Large Ship Manufacturing Time Efficiency I",
    "Standup M-Set Basic Medium Ship Manufacturing Material Efficiency I",
    "Standup M-Set Basic Medium Ship Manufacturing Time Efficiency I",
    "Standup M-Set Basic Small Ship Manufacturing Material Efficiency I",
    "Standup M-Set Basic Small Ship Manufacturing Time Efficiency I",
    "Standup M-Set Drone and Fighter Manufacturing Material Efficiency I",
    "Standup M-Set Drone and Fighter Manufacturing Time Efficiency I",
    "Standup M-Set Equipment Manufacturing Material Efficiency I",
    "Standup M-Set Equipment Manufacturing Time Efficiency I",
    "Standup M-Set Structure Manufacturing Material Efficiency I",
    "Standup M-Set Structure Manufacturing Time Efficiency I",
    "Standup M-Set Thukker Advanced Component Manufacturing Material Efficiency",
    "Standup M-Set Thukker Basic Capital Component Manufacturing Material Efficiency",
    "Standup M-Set Advanced Component Manufacturing Material Efficiency II",
    "Standup M-Set Advanced Component Manufacturing Time Efficiency II",
    "Standup M-Set Advanced Large Ship Manufacturing Material Efficiency II",
    "Standup M-Set Advanced Large Ship Manufacturing Time Efficiency II",
    "Standup M-Set Advanced Medium Ship Manufacturing Material Efficiency II",
    "Standup M-Set Advanced Medium Ship Manufacturing Time Efficiency II",
    "Standup M-Set Advanced Small Ship Manufacturing Material Efficiency II",
    "Standup M-Set Advanced Small Ship Manufacturing Time Efficiency II",
    "Standup M-Set Ammunition Manufacturing Material Efficiency II",
    "Standup M-Set Ammunition Manufacturing Time Efficiency II",
    "Standup M-Set Basic Capital Component Manufacturing Material Efficiency II",
    "Standup M-Set Basic Capital Component Manufacturing Time Efficiency II",
    "Standup M-Set Basic Large Ship Manufacturing Material Efficiency II",
    "Standup M-Set Basic Large Ship Manufacturing Time Efficiency II",
    "Standup M-Set Basic Medium Ship Manufacturing Material Efficiency II",
    "Standup M-Set Basic Medium Ship Manufacturing Time Efficiency II",
    "Standup M-Set Basic Small Ship Manufacturing Material Efficiency II",
    "Standup M-Set Basic Small Ship Manufacturing Time Efficiency II",
    "Standup M-Set Drone and Fighter Manufacturing Material Efficiency II",
    "Standup M-Set Drone and Fighter Manufacturing Time Efficiency II",
    "Standup M-Set Equipment Manufacturing Material Efficiency II",
    "Standup M-Set Equipment Manufacturing Time Efficiency II",
    "Standup M-Set Structure Manufacturing Material Efficiency II",
    "Standup M-Set Structure Manufacturing Time Efficiency II"
}

_AZBEL_RIGS = {
    "Standup L-Set Advanced Component Manufacturing Efficiency I",
    "Standup L-Set Advanced Large Ship Manufacturing Efficiency I",
    "Standup L-Set Advanced Medium Ship Manufacturing Efficiency I",
    "Standup L-Set Advanced Small Ship Manufacturing Efficiency I",
    "Standup L-Set Ammunition Manufacturing Efficiency I",
    "Standup L-Set Basic Capital Component Manufacturing Efficiency I",
    "Standup L-Set Basic Large Ship Manufacturing Efficiency I",
    "Standup L-Set Basic Medium Ship Manufacturing Efficiency I",
    "Standup L-Set Basic Small Ship Manufacturing Efficiency I",
    "Standup L-Set Capital Ship Manufacturing Efficiency I",
    "Standup L-Set Drone and Fighter Manufacturing Efficiency I",
    "Standup L-Set Equipment Manufacturing Efficiency I",
    "Standup L-Set Structure Manufacturing Efficiency I",
    "Standup L-Set Thukker Advanced Component Manufacturing Efficiency",
    "Standup L-Set Thukker Basic Capital Component Manufacturing Efficiency",
    "Standup L-Set Advanced Component Manufacturing Efficiency II",
    "Standup L-Set Advanced Large Ship Manufacturing Efficiency II",
    "Standup L-Set Advanced Medium Ship Manufacturing Efficiency II",
    "Standup L-Set Advanced Small Ship Manufacturing Efficiency II",
    "Standup L-Set Capital Ship Manufacturing Efficiency II",
    "Standup L-Set Ammunition Manufacturing Efficiency II",
    "Standup L-Set Basic Capital Component Manufacturing Efficiency II",
    "Standup L-Set Basic Large Ship Manufacturing Efficiency II",
    "Standup L-Set Basic Medium Ship Manufacturing Efficiency II",
    "Standup L-Set Basic Small Ship Manufacturing Efficiency II",
    "Standup L-Set Drone and Fighter Manufacturing Efficiency II",
    "Standup L-Set Equipment Manufacturing Efficiency II",
    "Standup L-Set Structure Manufacturing Efficiency II"
}

_SOTIYO_RIGS = {
    "Standup XL-Set Equipment and Consumable Manufacturing Efficiency I",
    "Standup XL-Set Ship Manufacturing Efficiency I",
    "Standup XL-Set Structure and Component Manufacturing Efficiency I",
    "Standup XL-Set Thukker Structure and Component Manufacturing Efficiency",
    "Standup XL-Set Equipment and Consumable Manufacturing Efficiency II",
    "Standup XL-Set Ship Manufacturing Efficiency II",
    "Standup XL-Set Structure and Component Manufacturing Efficiency II"
}


AVAIABLE_RIGS = _ATHANOR_RIGS | _TATARA_RIGS | _RAITARU_RIGS | _AZBEL_RIGS | _SOTIYO_RIGS



import json
import re

from django.shortcuts import render, get_object_or_404
from django.http import FileResponse
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.core.files.base import ContentFile

from allianceauth.services.hooks import get_extension_logger
from allianceauth.eveonline.models import EveCharacter

from .forms import ExportForm

logger = get_extension_logger(__name__)

skills_re = re.compile(r'^(?P<app>memberaudit|corptools)-(?P<character_id>\d+)$')


@login_required
def index(request):
    if request.method == 'POST':
        form = ExportForm(request.user, request.POST, request.FILES)
        if form.is_valid():
            error = False
            config = json.load(form.cleaned_data['config'])

            if form.cleaned_data['skills']:
                m = skills_re.match(form.cleaned_data['skills'])
                app = m.group('app')
                character = get_object_or_404(EveCharacter, character_id=int(m.group('character_id')), character_ownership__user=request.user)

                if app == 'memberaudit':
                    from .exporters.skills.memberaudit import import_skills, is_character_added

                    if not is_character_added(character):
                        messages.error(request, f"Character {character.character_name} is not added to MemberAudit")
                        error = True
                    else:
                        config.update(import_skills(character))
                elif app == 'corptools':
                    from .exporters.skills.corptools import import_skills, is_character_added

                    if not is_character_added(character):
                        messages.error(request, f"Character {character.character_name} is not added to CorpTools")
                        error = True
                    else:
                        config.update(import_skills(character))

            if form.cleaned_data['structures']:
                if 'structures' == form.cleaned_data['structures']:
                    from .exporters.structures.structures import export_structures
                elif 'corptools' == form.cleaned_data['structures']:
                    from .exporters.structures.corptools import export_structures
                else:
                    messages.error(request, "Invalid structures app")
                    error = True

                config['hidden_my_structures'] = export_structures(request.user)
                config['hidden_allocation_dict'] = {}

            if not error:
                updated_config = ContentFile(json.dumps(config, indent=4).encode())
                return FileResponse(updated_config, as_attachment=True, filename='config.json')

    else:
        form = ExportForm(request.user)

    context = {
        'form': form,
    }

    return render(request, 'ravworks_exporter/index.html', context=context)



from django import forms
from django.conf import settings
from django.db.models import Exists, OuterRef


class ExportForm(forms.Form):
    config = forms.FileField(required=True, label='Config file')
    skills = forms.ChoiceField(required=False, label='Skills', choices=[
        (None, 'Do not export'),
    ])
    structures = forms.ChoiceField(required=False, label='Structures', choices=[
        (None, 'Do not export'),
    ])

    def __init__(self, user, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'structures' in settings.INSTALLED_APPS:
            self.fields['structures'].choices = [*self.fields['structures'].choices, ('structures', 'Structures')]

        if 'memberaudit' in settings.INSTALLED_APPS and user.has_perm('memberaudit.basic_access'):
            from memberaudit.models import Character

            ownerships = (
                user.character_ownerships
                .filter(
                    Exists(
                        Character.objects.filter(
                            eve_character=OuterRef('character')
                        )
                    )
                )
                .select_related('character')
            )

            choices = [
                (f"memberaudit-{ownership.character.character_id}", f"MemberAudit - {ownership.character.character_name}")
                for ownership in ownerships
            ]

            self.fields['skills'].choices = [*self.fields['skills'].choices, *choices]

        if 'corptools' in settings.INSTALLED_APPS:
            from corptools.models import CharacterAudit

            ownerships = (
                user.character_ownerships
                .filter(
                    Exists(
                        CharacterAudit.objects.filter(
                            character=OuterRef('character')
                        )
                    )
                )
                .select_related('character')
            )

            choices = [
                (f"corptools-{ownership.character.character_id}", f"CorpTools - {ownership.character.character_name}")
                for ownership in ownerships
            ]

            self.fields['skills'].choices = [*self.fields['skills'].choices, *choices]

            self.fields['structures'].choices = [*self.fields['structures'].choices, ('corptools', 'CorpTools')]

        self.fields['skills'].initial = self.fields['skills'].choices[-1][0]
        self.fields['structures'].initial = self.fields['structures'].choices[-1][0]



from corptools.models import CorpAsset, Structure

from .structures_settings import AVAIABLE_RIGS


def export_structures(user):
    structures = (
        Structure
        .get_visible(user)
        .filter(type_name__group__category_id=65)
        .select_related('system_name')
    )
    res = []

    for i, structure in enumerate(structures, 1):
        rig_items = CorpAsset.get_visible(user).filter(
            location_flag__icontains="rig",
            location_id=structure.structure_id,
            type_name__name__in=AVAIABLE_RIGS,
        ).values_list('type_name__name', flat=True).distinct()

        rigs = {f"Rig{j}": "No Rig" for j in range(1, 4)}

        for j, rig in enumerate(rig_items, 1):
            rigs[f"Rig{j}"] = rig

        if round(structure.system_name.security_status, 1) >= 0.5:
            security = 'High'
        elif 0 < round(structure.system_name.security_status, 1) < 0.5:
            security = 'Low'
        else:
            security = 'Null / Wormhole'

        res.append({
            'id': f"Structure {i}",
            'name': structure.name,
            'security': security,
            'structure': structure.type_name.name,
            **rigs,
        })

    return res



from allianceauth.eveonline.models import EveCharacter

from memberaudit.models import CharacterSkill, Character

from .skill_settings import SKILL_SET


def import_skills(character: EveCharacter):
    skills = CharacterSkill.objects.filter(
        character__eve_character=character,
        eve_type_id__in=SKILL_SET,
    )

    res = {f"skill_{skill.eve_type_id}": skill.active_skill_level if skill.active_skill_level != 0 else 1 for skill in skills}

    missing_skills = SKILL_SET.difference(skills.values_list('eve_type_id', flat=True))
    res.update({f"skill_{skill_id}": 1 for skill_id in missing_skills})

    return res


def is_character_added(character: EveCharacter):
    return Character.objects.filter(eve_character=character).exists()



from allianceauth import hooks
from allianceauth.services.hooks import UrlHook, MenuItemHook

from . import urls


class RavworksExporterMenuItemHook(MenuItemHook):
    def __init__(self):
        super().__init__("Ravworks Exporter", "fas fa-wrench", "ravworks_exporter:index", navactive=['ravworks_exporter:'])


@hooks.register('menu_item_hook')
def register_menu():
    return RavworksExporterMenuItemHook()


@hooks.register('url_hook')
def register_urls():
    return UrlHook(urls, 'ravworks_exporter', 'ravworks_exporter/')
